/* Utility routines for remote SunOS kernel debugging.
   Copyright (C) 1993 by John Heidemann <johnh@cs.ucla.edu>

	@(#)kgdb.h	1.1 (John Heidemann) 9/12/94

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */



/*
 * A spinner on all I/O slows things down a lot, but it lets you know
 * you're not hung.
 */
/* #define USE_SPINNER */


/*
 * kiodebug enables verbose debugging printfs.
 * You probably don't want it on.
 */
/* #define USE_KIODEBUG */


/*
 * The kioecho serial i/o protocol requires that we echo each character
 * back that we get.  It's stupid and slow, but it helps if you
 * drop random characters during fast transmission.
 */
/* #define USE_KIOECHO */


/*
 * KGDB_DEVICE is the string we pass to the PROM to open our serial
 * port for I/O.  It is best determined by magic; Sun doesn't document
 * these things well.
 *
 * We set it to port B.
 */
#define KGDB_DEVICE "/zs@1,f1000000:b"



/*
 * NO_HARDWARE_HANDSHAKE -- Define if h/w handshaking unavailable.
 * Reads characters one at a time instead of en masse.
 * n-h-h is irrelevant if DIRECT_DRIVE is defined.
 */
#define NO_HARDWARE_HANDSHAKE


/*
 * If you want to be brave and drive the serial chip yourself, rather
 * than going through the PROMs, configure the following parameters.
 * The advantage of doing it this way is that the PROMs are *VERY*
 * slow, so that you can't run at 9600 baud, while driving the chip
 * directly is very fast and thus dramatically improves the
 * performance of kgdb.  The disadvantage, of course, is that you lose
 * a certain amount of portability.
 *
 */
#define DIRECT_DRIVE		/* Drive serial chip directly */


#ifdef DIRECT_DRIVE
/*
 * If you have a fast CPU and a slow serial chip, you may have to pick
 * a nonzero RECOVER_COUNT.
 */
#define RECOVER_COUNT	0

/*
 * If you define POLLCOUNT larger than 1, the console will not be
 * polled for input as frequently as the debugger port.  This may
 * improve reliability.  A good value is 500.
 */
#define POLLCOUNT	500

/*
 * DUMP_DESCRIPTORS prints out the PROM handles for the serial
 * devices.  It's for debugging and figuring out where your
 * serial ports really are.
 */
/* #define DUMP_DESCRIPTORS */

/*
 * The following values are the actual base addresses of the serial
 * chips in I/O space.  If you don't know what the correct values are,
 * and kgdb doesn't work, turn off DIRECT_DRIVE and turn on
 * DUMP_DESCRIPTORS.  Look through the output for something that looks
 * like an I/O base (it will probably be the third longword of the dump).
 */
#define REMOTE_BASE 0xffeeb000
#define STDIN_BASE 0xffeeb004
 
#endif /* DIRECT_DRIVE */




/*----------------------------------------------------------------------
 *
 * function prototypes
 *
 */

/* Stolen from 4.4BSD */
#if defined(__STDC__) || defined(__cplusplus)
#define __P(protos)     protos          /* full-blown ANSI C */
#else   /* !(__STDC__ || __cplusplus) */
#define __P(protos)     ()              /* traditional C preprocessor */
#endif

#if defined(__GNUC__) && !defined(__STRICT_ANSI__)
#define __dead __volatile
#else
#define __dead
#endif


/*
 * from kgdb_utils.h
 */
extern void perror __P((const char *s));
extern void panic __P((char *s));
extern void spinner __P((void));
extern void prespinner __P((void));
extern void postspinner __P((void));
extern void remote_open __P((void));
extern void remote_close __P((void));
extern void putpkt __P((char *buf));
extern void getpkt __P((char *buf));
extern void write_ok __P((char *buf));
extern void write_enn __P((char *buf));
extern void convert_int_to_ascii __P((char *from, char *to, int n));
extern void convert_ascii_to_int __P((char *from, char *to, int n));
extern void prepare_resume_reply __P((char *buf, unsigned char signal));
extern void prepare_long_resume_reply __P((char *buf, unsigned char signal, char *registers));
extern void decode_m_packet __P((char *from, unsigned int *mem_addr_ptr, unsigned int *len_ptr));
extern void decode_M_packet __P((char *from, unsigned int *mem_addr_ptr, unsigned int *len_ptr, char *to));
