/* Central loop for kgdb.
   Copyright (C) 1986, 1987 Free Software Foundation, Inc.
   Copyright (C) 1992,1993 by John Heidemann <johnh@cs.ucla.edu>

   This file is derived from rem-multi/remote_server.c from gdb 4.5.

	@(#)kgdb_loop.c	1.11 (John Heidemann) 11/19/93

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */


#include <setjmp.h>
#include <debug/debugger.h>
#include "defs.h"
#include "wait.h"
#include "patchlevel.h"


extern char registers[];

int reply_with_T = 1;

#define	PBUFSIZ	1024

void
kgdb_cmdloop()
{
	char own_buf[PBUFSIZ];
	char mem_buf[PBUFSIZ];
	char ch;
	int i;
	unsigned char signal;
	unsigned int mem_addr, len;
	char status;
	int send_reply_pkt = 1;
	extern int kiodebug;

	prespinner();
	for (;;) {
		getpkt(own_buf);
		if (kiodebug)
			printf("\nPacket received is <%s>\n", own_buf);
		i = 0;
		ch = own_buf[i++];
		switch (ch) {
		case 'g':
			kgdb_fetch_inferior_registers(-1);
			convert_int_to_ascii(registers, own_buf,
					     REGISTER_BYTES);
			break;
		case 'G':
			convert_ascii_to_int(&own_buf[1], registers,
					     REGISTER_BYTES);
			if (kgdb_store_inferior_registers(-1) == 0)
				write_ok(own_buf);
			else write_enn(own_buf);
			break;
		case 'm':
			decode_m_packet(&own_buf[1], &mem_addr, &len);
			kgdb_read_inferior_memory(mem_addr, mem_buf, len);
			convert_int_to_ascii(mem_buf, own_buf, len);
			break;
		case 'M':
			decode_M_packet(&own_buf[1], &mem_addr, &len,mem_buf);
			if (kgdb_write_inferior_memory(mem_addr, mem_buf, len) == 0)
				write_ok(own_buf);
			else
				write_enn(own_buf);
			break;
		case 'c':
		case 's':
			signal = kgdb_resume((ch == 's' ? 1 : 0), 0, &status);
			if (reply_with_T) {
				kgdb_fetch_inferior_registers(-1);
				prepare_long_resume_reply(own_buf, signal, registers);
			} else {
				prepare_resume_reply(own_buf, signal);
			};
			break;
		case '?':   /* starting message */
			signal = kgdb_resume(2,0,&status);
			prepare_resume_reply(own_buf, signal);
			break;
		case 3:   /* interrupt message */
			printf ("kgdb: sending interrupt reply.\n");
			prepare_resume_reply(own_buf, WSTOPSIG(SIGTRAP << 8));
			break;
		case 'k':
			kgdb_kill_inferior();
			own_buf[0] = 'q';
			own_buf[1] = 0;
			putpkt(own_buf);
			printf("\nObtained kill request...terminating\n");
			remote_close();
			break;
		case 'C':
			/*
			 * `C' is an interal packet; it should not really
			 * be sent by the remote client.
			 * It indicates that the user has requested
			 * to continue execution from the console.
			 * We obey this request by resuming.
			 * The only difference between handling
			 * a C packet and a c packet is that for a C
			 * packet we don't want to send a reply
			 * to the kgdb guy (who may not be there
			 * anyway).
			 */
			signal = kgdb_resume(0, 0, &status);
			send_reply_pkt = 0;
			break;
		default:
			printf("\nUnknown option chosen by master\n");
			write_enn(own_buf);
			break;
		};

		if (send_reply_pkt)
			putpkt(own_buf);
		send_reply_pkt = 1;
	};
#if 0
	panic("kgdb_cmdloop: exited loop!\n");
#endif
}


jmp_buf jb;



/*
 * We come here from spawn.
 * This routine is therefore the sole link between Sun's routines
 * and our kgdb stuff.  When invoked here (or returned
 * anywhere from doswitch) we expect regsave to contain
 * the registers of the inferior ``process'' at the
 * time of its demise.
 */
void
debugcmd()
{
	extern int kiodebug, kiospinner;

	printf ("kgdb release %sp%d\n", RELEASE, PATCHLEVEL);
	/*
	 * Spinning interfers with debugging messages.
	 * Debugging takes precedence.
	 */
	if (kiodebug)
		kiospinner = 0;
	remote_open();
	for (;;) {
		printf ("kgdb: waiting for L1-A.\n");
		doswitch();   /* go back to the real kernel */

		/* printf ("debugcmd: _longjmp to debugregs.\n"); */
		printf ("kgdb: waiting for debugger.\n");

		abort_jmp = jb;
		(void) _setjmp(jb);

		kgdb_cmdloop();
	}
}

